==> rx-0.1.1.tar.gz: lib/src/operators/sample.dart <==
library rx.operators.sample;

import '../constructors/timer.dart';
import '../core/observable.dart';
import '../core/observer.dart';
import '../core/subscriber.dart';
import '../disposables/disposable.dart';
import '../observers/inner.dart';
import '../schedulers/scheduler.dart';

extension SampleOperator<T> on Observable<T> {
  /// Emits the most recently emitted value from this [Observable] whenever the
  /// `trigger` emits.
  Observable<T> sample<R>(Observable<R> trigger) =>
      SampleObservable<T, R>(this, trigger);

  /// Emits the most recently emitted value from this [Observable] within
  /// periodic time intervals.
  Observable<T> sampleTime(Duration duration, {Scheduler? scheduler}) =>
      sample<int>(
          timer(delay: duration, period: duration, scheduler: scheduler));
}

class SampleObservable<T, R> extends Observable<T> {
  final Observable<T> delegate;
  final Observable<R> trigger;

  SampleObservable(this.delegate, this.trigger);

  @override
  Disposable subscribe(Observer<T> observer) {
    final subscriber = SampleSubscriber<T, R>(observer, trigger);
    subscriber.add(delegate.subscribe(subscriber));
    return subscriber;
  }
}

class SampleSubscriber<T, R> extends Subscriber<T>
    implements InnerEvents<R, void> {
  T? lastValue;
  bool hasLastValue = false;

  SampleSubscriber(Observer<T> observer, Observable<R> trigger)
      : super(observer) {
    add(InnerObserver(this, trigger, null));
  }

  @override
  void onNext(T value) {
    lastValue = value;
    hasLastValue = true;
  }

  @override
  void notifyNext(Disposable disposable, void state, R value) => emitValue();

  @override
  void notifyError(Disposable disposable, void state, Object error,
          StackTrace stackTrace) =>
      doError(error, stackTrace);

  @override
  void notifyComplete(Disposable disposable, void state) => emitValue();

  void emitValue() {
    if (hasLastValue) {
      doNext(lastValue);
      hasLastValue = false;
    }
  }
}

==> rx-0.1.2.tar.gz: lib/src/operators/sample.dart <==
import '../constructors/timer.dart';
import '../core/observable.dart';
import '../core/observer.dart';
import '../core/subscriber.dart';
import '../disposables/disposable.dart';
import '../observers/inner.dart';
import '../schedulers/scheduler.dart';

extension SampleOperator<T> on Observable<T> {
  /// Emits the most recently emitted value from this [Observable] whenever the
  /// `trigger` emits.
  Observable<T> sample<R>(Observable<R> trigger) =>
      SampleObservable<T, R>(this, trigger);

  /// Emits the most recently emitted value from this [Observable] within
  /// periodic time intervals.
  Observable<T> sampleTime(Duration duration, {Scheduler? scheduler}) =>
      sample<int>(
          timer(delay: duration, period: duration, scheduler: scheduler));
}

class SampleObservable<T, R> with Observable<T> {
  final Observable<T> delegate;
  final Observable<R> trigger;

  SampleObservable(this.delegate, this.trigger);

  @override
  Disposable subscribe(Observer<T> observer) {
    final subscriber = SampleSubscriber<T, R>(observer, trigger);
    subscriber.add(delegate.subscribe(subscriber));
    return subscriber;
  }
}

class SampleSubscriber<T, R> extends Subscriber<T>
    implements InnerEvents<R, void> {
  T? lastValue;
  bool hasLastValue = false;

  SampleSubscriber(Observer<T> observer, Observable<R> trigger)
      : super(observer) {
    add(InnerObserver<R, void>(this, trigger, null));
  }

  @override
  void onNext(T value) {
    lastValue = value;
    hasLastValue = true;
  }

  @override
  void notifyNext(Disposable disposable, void state, R value) => emitValue();

  @override
  void notifyError(Disposable disposable, void state, Object error,
          StackTrace stackTrace) =>
      doError(error, stackTrace);

  @override
  void notifyComplete(Disposable disposable, void state) => emitValue();

  void emitValue() {
    if (hasLastValue) {
      doNext(lastValue);
      hasLastValue = false;
    }
  }
}
